import com.fishlib.auth.SimpleUserContext;
import com.fishlib.configuration.Configuration;
import com.fishlib.io.logger.Logger;
import com.fishlib.io.logger.StreamLoggerImpl;
import com.illumon.iris.db.schema.SchemaService;
import com.illumon.iris.db.schema.SchemaServiceFactory;
import com.illumon.iris.db.tables.Table;
import com.illumon.iris.db.tables.databases.Database;
import com.illumon.iris.db.tables.databases.OnDiskQueryDatabase;
import com.illumon.iris.db.tables.utils.TableTools;
import com.illumon.iris.db.v2.locations.TableDataService;
import com.illumon.iris.db.v2.locations.local.LocalTableDataService;
import com.illumon.iris.db.v2.locations.local.LocalTablePathManager;
import com.illumon.iris.db.v2.permissions.PermissionFilterProvider;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;

import java.io.File;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;

public class UnitTest {

    //logs go to stdout
    private static final Logger logger = new StreamLoggerImpl();
    private static final String TEST_DATE = "2017-08-25";

    private static Database getTestDatabase() throws IOException {
        //create a local SchemaService
        final Configuration configuration = Configuration.getInstance();
        final String schemaDir = "src/test/resources";
        final SchemaService schemaService = SchemaServiceFactory.getLocalInstance(configuration, schemaDir);

        //create the OnDiskQueryDatabase
        final Path rootPath = Files.createTempDirectory(Paths.get(Configuration.getInstance().getWorkspacePath()), "UnitTest");
        final File rootFile = rootPath.toFile();
        final LocalTablePathManager pathManager = new LocalTablePathManager(rootFile);
        final TableDataService tableDataService = new LocalTableDataService(pathManager);
        final OnDiskQueryDatabase db = new OnDiskQueryDatabase(logger, rootFile, tableDataService, schemaService);

        //start the OnDiskQueryDatabase and set a user context
        db.startup();
        Configuration.getInstance().setProperty("IrisDB.permissionFilterProvider", "null");
        PermissionFilterProvider.FACTORY.reload();
        db.setUserContext(null, new SimpleUserContext("nobody", "nobody"));

        //add the test data
        addData(db);
        return db;
    }

    private static void addData(final Database db) throws IOException {
        final Table t = TableTools.readCsv(new File("src/test/resources/StockTrades.csv"));
        db.addTablePartition("Example", "StockTrades", TEST_DATE, t);
    }

    @Test
    public void testQuery() throws IOException {
        final Database db = getTestDatabase();

        final Table testResult = QueryUtils.getAvgStockPriceTable(db, TEST_DATE);

        final double sumAAPL = testResult.where("USym = `AAPL`").getColumn("Last").getDouble(0);
        Assertions.assertEquals(159.472, sumAAPL, 0.0002);

        final double sumGOOG = testResult.where("USym = `GOOG`").getColumn("Last").getDouble(0);
        Assertions.assertEquals(923.56, sumGOOG, 0.0002);
    }

}